
# q-expansion of newform 845.2.e.g, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 845
weight = 2
poly_data = [1, 1, 2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[1, 0, 0, 1], 2], [[-1, -2, 2, -1], 2]]

hecke_ring_character_values = [[677, [1, 0, 0, 0]], [171, [0, 0, 0, 1]]]
aps_data = [[0, 1, 0, 0], [-1, 2, 0, -1], [-1, 0, 0, 0], [0, 2, 2, -3], [1, 2, 0, 1], [0, 0, 0, 0], [0, 4, 4, -1], [0, 2, 2, -3], [-5, -2, 0, -5], [5, -4, 0, 5], [0, 0, 0, 0], [3, 0, 0, 3], [7, -8, 0, 7], [0, 2, 2, -5], [0, 0, 8, 0], [6, 0, 0, 0], [0, -6, -6, -3], [0, -12, -12, 7], [1, 6, 0, 1], [0, -2, -2, 5], [6, 0, 0, 0], [0, 0, 0, 0], [-4, 0, -8, 0], [-9, 0, 0, -9], [0, -4, -4, 1], [-3, -4, 0, -3], [8, 0, 8, 0], [7, 2, 0, 7], [2, 0, 0, 0], [0, -4, -4, -1], [3, -2, 0, 3], [-12, 0, 0, 0], [0, 4, 4, 1], [0, 6, 6, -13], [0, -4, -4, -7], [-8, 0, 0, 0], [-18, 0, 0, 0], [0, 6, 6, -11], [-11, 10, 0, -11], [0, 16, 16, -9], [7, -2, 0, 7], [6, 0, 0, 0], [0, 10, 10, -21], [-1, 4, 0, -1], [3, 0, 0, 3], [0, -6, -6, -5], [7, -10, 0, 7], [-3, -6, 0, -3], [0, -2, -2, 5], [-6, 0, -16, 0], [-6, 0, -16, 0], [16, 0, 16, 0], [0, 0, 0, -15], [0, -2, -2, 19], [-15, 4, 0, -15], [-5, -2, 0, -5], [0, -4, -4, -9], [5, 6, 0, 5], [0, -16, -16, 7], [6, 0, 16, 0], [7, -6, 0, 7], [0, -8, -8, 25], [-20, 0, -8, 0], [-24, 0, 0, 0], [-6, 0, 16, 0], [2, 0, 16, 0], [0, -14, -14, -3], [2, 0, -16, 0], [0, -6, -6, 27], [-5, -12, 0, -5], [-17, 4, 0, -17], [-8, 0, -16, 0], [3, 14, 0, 3], [0, -8, -8, 23], [17, 10, 0, 17], [0, 2, 2, -3], [-10, 0, 16, 0], [0, 8, 8, -23], [-1, -16, 0, -1], [0, 16, 16, -15], [-9, 14, 0, -9], [-6, 0, 0, 0], [21, -10, 0, 21], [0, -4, -4, -17], [19, -6, 0, 19], [-4, 0, 8, 0], [0, 8, 8, 1], [-9, -12, 0, -9], [0, 12, 12, 9], [-16, 0, 8, 0], [4, 0, 8, 0], [21, -18, 0, 21], [0, 18, 18, -3], [-1, -2, 0, -1], [12, 0, 16, 0], [0, -10, -10, 3], [-21, 20, 0, -21], [10, 0, 32, 0], [-9, -6, 0, -9], [18, 0, 16, 0], [4, 0, 24, 0], [-5, 8, 0, -5], [0, -6, -6, 27], [-7, 16, 0, -7], [28, 0, 0, 0], [-18, 0, -16, 0], [9, -2, 0, 9], [-2, 0, 16, 0], [-24, 0, 0, 0], [-39, 8, 0, -39], [0, 6, 6, 3], [35, -8, 0, 35], [0, -12, -12, 33], [-12, 0, 0, 0], [0, -18, -18, 5], [0, -16, -16, -1], [0, 6, 6, 21], [11, 22, 0, 11], [21, 0, 0, 21], [0, -2, -2, -45], [-29, 20, 0, -29], [17, 12, 0, 17], [22, 0, -16, 0], [0, -2, -2, -27], [25, -22, 0, 25], [-2, 0, 16, 0], [0, 28, 28, -7], [0, 18, 18, -21], [24, 0, 8, 0], [34, 0, 16, 0], [9, -30, 0, 9], [-27, 18, 0, -27], [27, -6, 0, 27], [-3, -16, 0, -3], [0, -8, -8, 33], [15, -24, 0, 15], [0, 8, 8, 25], [0, 30, 30, -11], [0, -24, -24, 39], [-7, -16, 0, -7], [-12, 0, -32, 0], [-13, 20, 0, -13], [0, 22, 22, -13], [4, 0, 8, 0], [21, -36, 0, 21], [0, 6, 6, -27], [-6, 0, 0, 0], [26, 0, 16, 0], [12, 0, 16, 0], [0, 0, -24, 0], [0, -40, -40, 9], [-15, 0, 0, -15], [-28, 0, -24, 0], [-21, 38, 0, -21], [-9, 18, 0, -9], [48, 0, 0, 0], [0, -6, -6, 43], [0, 8, 8, -15], [-6, 0, 0, 0], [2, 0, 16, 0], [33, -10, 0, 33], [0, -44, -44, 15], [40, 0, 24, 0], [0, -2, -2, -13], [15, 12, 0, 15], [-24, 0, -24, 0], [43, -6, 0, 43], [0, 16, 16, -9]]
